"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const _ = require("lodash");
const command_2 = require("../../command");
const cf = require("../../utils/common-flags");
const lazy_1 = require("../../utils/lazy");
const oclif_utils_1 = require("../../utils/oclif-utils");
const version_1 = require("../../utils/version");
const deprecatedInfo = version_1.isV13()
    ? ''
    : `
The --verbose option may add extra columns/fields to the output. Currently
this includes the "STATE" column which is DEPRECATED and whose values are one
of 'new', 'released' or 'discontinued'. However, 'discontinued' device types
are only listed if the '--discontinued' option is also used, and this option
is also DEPRECATED.
`
        .split('\n')
        .join(`\n\t\t`);
class DevicesSupportedCmd extends command_2.default {
    async run() {
        var _a, _b;
        const { flags: options } = this.parse(DevicesSupportedCmd);
        const [dts, configDTs] = await Promise.all([
            lazy_1.getBalenaSdk().models.deviceType.getAllSupported({
                $expand: { is_of__cpu_architecture: { $select: 'slug' } },
                $select: ['slug', 'name'],
            }),
            lazy_1.getBalenaSdk().models.config.getDeviceTypes(),
        ]);
        const dtsBySlug = _.keyBy(dts, (dt) => dt.slug);
        const configDTsBySlug = _.keyBy(configDTs, (dt) => dt.slug);
        const discontinuedDTs = version_1.isV13()
            ? []
            : configDTs.filter((dt) => dt.state === 'DISCONTINUED');
        const discontinuedDTsBySlug = _.keyBy(discontinuedDTs, (dt) => dt.slug);
        const slugsOfInterest = new Set([
            ...Object.keys(dtsBySlug),
            ...Object.keys(discontinuedDTsBySlug),
        ]);
        let deviceTypes = [];
        for (const slug of slugsOfInterest) {
            const configDT = configDTsBySlug[slug] || {};
            if (configDT.state === 'DISCONTINUED' && !options.discontinued) {
                continue;
            }
            const dt = dtsBySlug[slug] || {};
            const aliases = (configDT.aliases || []).filter((alias) => alias !== slug);
            deviceTypes.push({
                slug,
                aliases: options.json ? aliases : [aliases.join(', ')],
                arch: ((_b = (_a = dt.is_of__cpu_architecture) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.slug) ||
                    configDT.arch ||
                    'n/a',
                state: version_1.isV13()
                    ? undefined
                    : (configDT.state || 'NEW').replace('BETA', 'NEW'),
                name: dt.name || configDT.name || 'N/A',
            });
        }
        const fields = options.verbose && !version_1.isV13()
            ? ['slug', 'aliases', 'arch', 'state', 'name']
            : ['slug', 'aliases', 'arch', 'name'];
        deviceTypes = _.sortBy(deviceTypes, fields);
        if (options.json) {
            console.log(JSON.stringify(deviceTypes, null, 4));
        }
        else {
            const visuals = lazy_1.getVisuals();
            const output = await visuals.table.horizontal(deviceTypes, fields);
            console.log(output);
        }
    }
}
exports.default = DevicesSupportedCmd;
DevicesSupportedCmd.description = lazy_1.stripIndent `
		List the supported device types (like 'raspberrypi3' or 'intel-nuc').

		List the supported device types (like 'raspberrypi3' or 'intel-nuc').
		${deprecatedInfo}
		The --json option is recommended when scripting the output of this command,
		because the JSON format is less likely to change and it better represents data
		types like lists and empty strings (for example, the ALIASES column contains a
		list of zero or more values). The 'jq' utility may be helpful in shell scripts
		(https://stedolan.github.io/jq/manual/).
`;
DevicesSupportedCmd.examples = [
    '$ balena devices supported',
    '$ balena devices supported --verbose',
    '$ balena devices supported -vj',
];
DevicesSupportedCmd.usage = ('devices supported ' +
    new oclif_utils_1.CommandHelp({ args: DevicesSupportedCmd.args }).defaultUsage()).trim();
DevicesSupportedCmd.flags = {
    discontinued: command_1.flags.boolean({
        description: version_1.isV13()
            ? 'No effect (DEPRECATED)'
            : 'include "discontinued" device types (DEPRECATED)',
    }),
    help: cf.help,
    json: command_1.flags.boolean({
        char: 'j',
        description: 'produce JSON output instead of tabular output',
    }),
    verbose: command_1.flags.boolean({
        char: 'v',
        description: version_1.isV13()
            ? 'No effect (DEPRECATED)'
            : 'add extra columns in the tabular output (DEPRECATED)',
    }),
};
//# sourceMappingURL=supported.js.map