"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.whichSpawn = exports.which = exports.whichBin = exports.sanitizePath = exports.exists = exports.X_OK = exports.W_OK = exports.R_OK = exports.F_OK = void 0;
const fs_1 = require("fs");
const path = require("path");
exports.F_OK = fs_1.constants.F_OK, exports.R_OK = fs_1.constants.R_OK, exports.W_OK = fs_1.constants.W_OK, exports.X_OK = fs_1.constants.X_OK;
async function exists(filename, mode = exports.F_OK) {
    try {
        await fs_1.promises.access(filename, mode);
        return true;
    }
    catch (_a) {
        return false;
    }
}
exports.exists = exists;
function sanitizePath(filepath) {
    const filenamify = require('filenamify');
    return path
        .normalize(filepath)
        .split(path.sep)
        .map((f) => filenamify(f, { replacement: '_' }))
        .join(path.sep);
}
exports.sanitizePath = sanitizePath;
async function whichBin(programName) {
    for (const dir of ['/usr/bin', '/bin', '/usr/sbin', '/sbin']) {
        const candidate = path.join(dir, programName);
        if (await exists(candidate, exports.X_OK)) {
            return candidate;
        }
    }
    return '';
}
exports.whichBin = whichBin;
async function which(program, rejectOnMissing = true) {
    const whichMod = await Promise.resolve().then(() => require('which'));
    let programPath;
    try {
        programPath = await whichMod(program);
    }
    catch (err) {
        if (err.code === 'ENOENT') {
            if (rejectOnMissing) {
                const { ExpectedError } = await Promise.resolve().then(() => require('../errors'));
                throw new ExpectedError(`'${program}' program not found. Is it installed?`);
            }
            else {
                return '';
            }
        }
        throw err;
    }
    return programPath;
}
exports.which = which;
async function whichSpawn(programName, args, options = { stdio: 'inherit' }, returnExitCodeOrSignal = false) {
    const { spawn } = await Promise.resolve().then(() => require('child_process'));
    const program = await which(programName);
    if (process.env.DEBUG) {
        console.error(`[debug] [${program}, ${args.join(', ')}]`);
    }
    let error;
    let exitCode;
    let exitSignal;
    try {
        [exitCode, exitSignal] = await new Promise((resolve, reject) => {
            spawn(program, args, options)
                .on('error', reject)
                .on('close', (code, signal) => resolve([code, signal]));
        });
    }
    catch (err) {
        error = err;
    }
    if (error || (!returnExitCodeOrSignal && (exitCode || exitSignal))) {
        const msg = [
            `${programName} failed with exit code=${exitCode} signal=${exitSignal}:`,
            `[${program}, ${args.join(', ')}]`,
            ...(error ? [`${error}`] : []),
        ];
        throw new Error(msg.join('\n'));
    }
    return [exitCode, exitSignal];
}
exports.whichSpawn = whichSpawn;
//# sourceMappingURL=which.js.map